"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStreamRouter = void 0;
const pump_1 = __importDefault(require("pump"));
const range_parser_1 = __importDefault(require("range-parser"));
const express_1 = require("express");
const http_errors_1 = require("http-errors");
const querystring_1 = require("querystring");
const torrent_client_1 = require("../services/torrent-client");
const helpers_1 = require("../helpers");
const validation_1 = require("../helpers/validation");
function getStreamRouter(config, _logger, client) {
    return express_1.Router()
        .get('/stream/:torrent(*)', async (req, res) => {
        const encodeToken = config.security.streamApi.key || config.security.apiKey;
        // If security is enabled, require encoded torrent with JWT
        const data = encodeToken
            ? helpers_1.verifyJwtToken(req.params.torrent, encodeToken, config.security.streamApi.maxAge)
            : {
                ...req.query,
                torrent: req.params.torrent,
            };
        if (!data) {
            throw new http_errors_1.Forbidden('Incorrect JWT encoding');
        }
        if (encodeToken && Object.values(req.query).some((v) => !!v)) {
            throw new http_errors_1.BadRequest(`All parameters should be encoded with JWT`);
        }
        const torrent = await client.addTorrent(validation_1.validateString(data.torrent, 'torrent'));
        const file = torrent_client_1.findFile(torrent.files, {
            file: data.file ? validation_1.validateString(data.file, 'file') : undefined,
            fileIndex: data.fileIndex
                ? validation_1.validateInt(data.fileIndex, 'fileIndex')
                : undefined,
            fileType: data.fileType ? validation_1.validateString(data.fileType, 'fileType') : undefined,
        });
        if (!file) {
            throw new http_errors_1.NotFound();
        }
        res.statusCode = 200;
        res.setHeader('Accept-Ranges', 'bytes');
        res.attachment(file.name);
        req.connection.setTimeout(3600000);
        const parsedRange = req.headers.range
            ? range_parser_1.default(file.length, req.headers.range)
            : undefined;
        const range = parsedRange instanceof Array ? parsedRange[0] : undefined;
        if (range) {
            res.statusCode = 206;
            res.setHeader('Content-Length', range.end - range.start + 1);
            res.setHeader('Content-Range', 'bytes ' + range.start + '-' + range.end + '/' + file.length);
        }
        else {
            res.setHeader('Content-Length', file.length);
        }
        if (req.method === 'HEAD') {
            return res.end();
        }
        return pump_1.default(file.createReadStream(range), res, () => {
            file.stop();
        });
    })
        .get('/stream', async (req, res) => {
        const torrent = 'token' in req.query
            ? validation_1.validateString(req.query.token, 'token')
            : validation_1.validateString(req.query.torrent, 'torrent');
        return res.redirect(`/stream/${encodeURIComponent(torrent)}?${querystring_1.stringify({
            file: 'file' in req.query ? validation_1.validateString(req.query.file, 'file') : undefined,
            fileIndex: 'fileIndex' in req.query
                ? validation_1.validateInt(req.query.fileIndex, 'fileIndex')
                : undefined,
            fileType: 'fileType' in req.query
                ? validation_1.validateString(req.query.fileType, 'fileType')
                : undefined,
        })}`, 301);
    });
}
exports.getStreamRouter = getStreamRouter;
