import configSchema from './config.schema.json';
declare type DeepPartial<T> = {
    [P in keyof T]?: DeepPartial<T[P]>;
};
/**
 * Project Enviroment variables.
 *
 * Configuration file can overwrite ENV variables.
 */
interface EnvVariables {
    /**
     * Project environment.
     *
     * Default:  `production`
     */
    NODE_ENV: 'development' | 'production';
    /**
     * Server host.
     *
     * Default:  `0.0.0.0`
     */
    HOST: string;
    /**
     * Server port.
     *
     * Default:  `env.PORT` or 3000
     */
    PORT: number;
    /**
     * Get ip from `X-Forwarded-*` header.
     *
     * Default:  true if inside App Engine or Heroku else false
     */
    TRUST_PROXY: boolean;
    /**
     * Protect API with key. It should be passed to headers to access the API (`authorization`: `bearer ${apiKey}`).
     *
     * If `streamApi` doesn't have a separate key, `apiKey` can be used as JSON Web Token.
     *
     * Default:  undefined
     */
    API_KEY: string;
    /**
     * JSON config
     */
    CONFIG: DeepPartial<Config>;
}
/**
 * Configuration file interface, it can overwrite ENV variables.
 */
export interface Config {
    /**
     * Server host.
     *
     * Default:  `0.0.0.0`
     */
    host: string;
    /**
     * Server port.
     *
     * Default:  `env.PORT` or 3000
     */
    port: number;
    /**
     * Project environment.
     *
     * Default:  `production`
     */
    environment: 'development' | 'production';
    /**
     * Logging configuration.
     *
     * Default:  `console`
     */
    logging: {
        transports: Array<{
            /**
             * Log to console
             */
            type: 'console';
        } | {
            /**
             * Enable [loggly](https://www.loggly.com/) logging
             */
            type: 'loggly';
            /**
             * Loggly subdomain
             */
            subdomain: string;
            /**
             * Loggly token
             */
            token: string;
            /**
             * Loggly tags, can be used to filter logs
             */
            tags?: string[];
        }>;
        /**
         * Logging level.
         *
         * Default:  `info`
         */
        level: 'debug' | 'info' | 'warn' | 'error';
    };
    /**
     * Torrent client settings
     */
    torrents: {
        path: string;
        /**
         * Delete torrent data if it is inactive for X seconds.
         *
         * Default:  60 * 60
         */
        ttl: number;
        /**
         * Load defaul trackers list & use it for each torrents.
         *
         * Default:  true
         */
        useDefaultTrackers: boolean;
        /**
         * Additional trackers (`tr` parameter) which will be appened to each torrent.
         *
         * Default:  []
         */
        announce: string[];
        /**
         * Web Seed (`ws` parameter) which will be appened to each torrent.
         *
         * Default:  []
         */
        urlList: string[];
        /**
         * Peer addresses (`x.pe` parameter).
         *
         * Default:  []
         */
        peerAddresses: string[];
    };
    /**
     * Security settings
     */
    security: {
        /**
         * Stream API (`/stream/:torrent`) settings
         */
        streamApi: {
            /**
             * Protect stream API with [JSON Web Token](https://jwt.io/).
             *
             * If `apiKey` is not set, it can also be used as API key (`authorization`: `bearer ${apiKey}`).
             *
             * Default:  undefined
             */
            key?: string;
            /**
             * The maximum allowed age for tokens to still be valid.
             * It is expressed in seconds or a string describing a time span [zeit/ms](https://github.com/vercel/ms)
             *
             * Default:  `6h`
             */
            maxAge: string;
        };
        /**
         * Serve frontend static files
         *
         * Default:  environment === 'production'
         */
        frontendEnabled: boolean;
        /**
         * Enable API
         *
         * Default:  true
         */
        apiEnabled: boolean;
        /**
         * Protect API with key. It should be passed to headers to access the API (`authorization`: `bearer ${apiKey}`).
         *
         * If `streamApi` doesn't have a separate key, `apiKey` can be used as JSON Web Token.
         *
         * Default:  undefined
         */
        apiKey?: string;
    };
    /**
     * Get ip from `X-Forwarded-*` header.
     *
     * Default:  true if inside App Engine or Heroku else false
     */
    trustProxy: boolean;
}
/**
 * Load enviroment variable using `EnvVariables` interface
 */
export declare function getEnv<T extends keyof EnvVariables>(key: T, type?: 'json' | 'int' | 'string' | 'bool'): EnvVariables[T] | undefined;
export declare const isInGoogleAppEngine: boolean;
export declare const isInHeroku: boolean;
export declare function readConfig(path: string | undefined): Promise<Config>;
/**
 * Frontend package path to build directory
 */
export declare const frontendBuildPath: string;
export { configSchema };
